package main

import (
	"encoding/json"
	"errors"
	"log"
	"net"
)

func client(conn net.Conn, instance string) error {
	log.Println("Client connected")
	buffer := make([]byte, 1024)
	n, err := conn.Read(buffer)
	message := string(buffer[:n])
	if err != nil {
		log.Fatal(err)
	}
	log.Println("Message Received:", message)
	var objmap map[string]interface{}
	err = json.Unmarshal([]byte(message), &objmap)
	if err != nil {
		log.Println(err)
		return err
	}
	val, has := objmap["command"]
	if has {
		switch val {
		case "run":
			_, err = runMap(conn, instance, objmap)
			if err != nil {
				log.Println(err)
				return err
			}
		case "remove":
			err = removeMap(conn, instance, objmap)
			if err != nil {
				log.Println(err)
				return err
			}
		case "list":
			err = list(conn, instance)
			if err != nil {
				log.Println(err)
				return err
			}
		default:
			err = errors.New("Unknown Command: " + val.(string))
			log.Println(err)
			return err
		}
	}
	return nil
}
