package main

import (
	"encoding/json"
	"log"
	"net"
	"os"
	"path/filepath"
)

func list(conn net.Conn, instance string) error {

	package_list := make(map[string]string)

	packages_path, err := filepath.Abs(filepath.Join(instance, "apps"))
	if err != nil {
		return err
	}

	packages, err := os.ReadDir(packages_path)
	if err != nil {
		return err
	}

	for _, program := range packages {
		if program.Type().IsDir() {
			info_path, err := filepath.Abs(filepath.Join(packages_path, program.Name(), "boundaries.json"))
			if err != nil {
				log.Println(err)
				continue
			}
			if exists, err := path_exists(info_path); exists && err == nil {
				infofile_content, err := os.ReadFile(info_path)
				if err != nil {
					log.Println(err)
					continue
				}
				var info map[string]interface{}
				err = json.Unmarshal(infofile_content, &info)
				if err != nil {
					log.Println(err)
					continue
				}

				version, has := info["version"]
				if !has {
					// setting a version is technically not required
					version = ""
				}
				package_list[program.Name()] = version.(string)
			}
		}
	}

	data, err := json.Marshal(package_list)
	if err != nil {
		return err
	}

	conn.Write(data)

	return nil
}
